import uuid

identifier = str(uuid.uuid4()).replace("-", "")

example="""ai_environment "AgentMicroservice" {
    architecture{
        envid = "fb98001a0ce94c44ad091de3d2e78164"
        service-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:service-layer-fastapi-base
        ai-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:ai-layer-pydanticai
        data-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:data-layer-pydantic
        agentic-middleware-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:agentic-middleware-layer-redistream
    }
    data{
        entity "Restaurant Query" {
            uid = aiurn:entity:id:restaurantquery
            element = aiurn:entity:var:question -> TEXT # "The question to the metre"
        }
        entity "Restaurant Answer" {
            uid = aiurn:entity:id:restaurantanswer 
            element = aiurn:entity:var:answer -> TEXT # "The answer of the metre"
            element = aiurn:entity:var:restaurant -> aiurn:entity:id:restaurant  # "The restaurant of the metre"
        }
        entity "Restaurant" {
            uid = aiurn:entity:id:restaurant 
            element = aiurn:entity:var:name -> TEXT # "The name of the restaurant"
            element = aiurn:entity:var:street -> TEXT # "The street where the restaurant is located"
            element = aiurn:entity:var:city -> TEXT # "The city where the restaurant is located"
            element = aiurn:entity:var:rating -> NUMBER # "The rating of the restaurant"
        }

        entity "BMI Query" {
            uid = aiurn:entity:id:bmiquery
            element = aiurn:entity:var:weight -> NUMBER # "The current weight of the person"
            element = aiurn:entity:var:height -> NUMBER # "The current height of the person in meters"
        }

        entity "BMI Result" {
            uid = aiurn:entity:id:bmiresult
            element = aiurn:entity:var:bmi -> NUMBER # "The calcualted bmi of the person"
        }
    }

    infrastructure {
        llm "My LLM" {
            uid = aiurn:model:id:geepeetee
            provider = aiurn:model:provider:azure 
            model = "gpt-4o"
            endpoint = "https://any.openai.azure.com/"
            version = "2025-01-01-preview"
            aiurn:global:var:temperature = 0.7
            aiurn:global:var:costid = "ewe3949" 
            aiurn:global:var:hello = True 
        }
    }


    functional{
        agent "Cook" {
            uid = aiurn:agent:id:cook
            namespace = aiurn:ns:moewe:kitchen
            systemprompt = "You're a four star rated metre working at restaurant https://moewe.agenterprise.ai/"
            llmref = aiurn:model:id:geepeetee 
            description = "A Cook in a restaurant"
            example = "Hey, there is a fly in my soup"
            example = "Delicious, hmmm"
            tag = "Restaurant"
            tag = "Recipies"
            toolref = aiurn:tool:id:crawler:v2
            in = aiurn:entity:id:restaurantquery
            out = aiurn:entity:id:restaurantanswer 
            aiurn:global:var:name = "Max Mustermann"
            aiurn:global:var:role = "cook"
            aiurn:global:var:lifeycle = "permanent"
            aiurn:global:var:events = "onRestaurantOpening"
          
        }

        agent "Waiter" {
            uid = aiurn:agent:id:waiter
            namespace = aiurn:ns:moewe:guestroom
            systemprompt = "Du bist eine freundliche und aufmerksame Serviekraft und arbeitest im  Restaurant https://moewe.agenterprise.ai/"
            llmref = aiurn:model:id:geepeetee 
            description = "A Waiter in a restaurant"
            example = "One drink, please"
            example = "Delicious, hmmm"
            tag = "Restaurant"
            tag = "Service"
            toolref = aiurn:tool:id:bmi:v1
            toolref = aiurn:tool:id:crawler:v2
            aiurn:global:var:name = "Max Mustermann"
            aiurn:global:var:role = "waiter"
            aiurn:global:var:lifeycle = "permanent"
            aiurn:global:var:events = "onRestaurantOpening"
        }
        tool "bmicalculator" {
            uid = aiurn:tool:id:bmi:v1
            in = aiurn:entity:id:bmiquery
            out = aiurn:entity:id:bmiresult
            endpoint = "lambda bmiquery: ToolOutputType(bmi=round(bmiquery.weight / (bmiquery.height ** 2), 2))"
            type = aiurn:tooltype:code
            description = "Tool calculating the bmi by weight and height"
            
        }
        
         tool "Webcrawler" {
            uid = aiurn:tool:id:crawler:v2
            endpoint = "https://remote.mcpservers.org/fetch/mcp"
            type = aiurn:tooltype:mcp
            description = "Tool for fetching webpages"
            
        }

    }
}""".replace("envid = \"fb98001a0ce94c44ad091de3d2e78164\"","envid = \""+identifier+"\"")



from agenterprise.agent_grammer.parser.ai_environmentListener import  ai_environmentListener
from agenterprise.model.data.ai_environment import Agent
from agenterprise.model.data.ai_environment import LLM
from agenterprise.model.listener.AIURN import AIURN

class BaseAILLMListener(ai_environmentListener):
    def __init__(self):
        super().__init__()
        self.llms = []
        self.current_llm = None

    def enterLlmDef(self, ctx):
        super().enterLlmDef(ctx)
        self.current_llm = {
            "name": ctx.PROPERTYVALUE().getText(),
            "properties" : {
               
            }
        
        }

    def enterLlmIdProp(self, ctx):
        super().enterLlmIdProp(ctx)
        self.current_llm["uid"] = ctx.LLMID().getText()
    def enterLlmEndpointProp(self, ctx):
        super().enterLlmEndpointProp(ctx)
        self.current_llm["endpoint"] = ctx.PROPERTYVALUE().getText()

    def enterLlmModelProp(self, ctx):
        super().enterLlmModelProp(ctx)
        self.current_llm["model"] = ctx.PROPERTYVALUE().getText()

    def enterLlmVersionProp(self, ctx):
        super().enterLlmVersionProp(ctx)   
        self.current_llm["version"] = ctx.PROPERTYVALUE().getText()

    def enterLlmProviderProp(self, ctx):
        super().enterLlmProviderProp(ctx) 
        self.current_llm["provider"] = ctx.LLMPROVIDER().getText() 

    def enterLlmOtherProperty(self, ctx):
        super().enterLlmOtherProperty(ctx)
        var_name = ctx.VAR().getText()
        var_value = ctx.PROPERTYVALUE().getText() 
        
        self.current_llm["properties"][AIURN(var_name)] = var_value
    

    def exitLlmDef(self, ctx):
        super().exitLlmDef(ctx)
        llm = LLM(
            name=self.current_llm.get("name"),
            uid=AIURN(self.current_llm.get("uid")),
            provider=AIURN(self.current_llm.get("provider")),
            model=self.current_llm.get("model"),
            endpoint=self.current_llm.get("endpoint"),
            version=self.current_llm.get("version"),
            properties=self.current_llm.get("properties")
        )
        self.llms.append(llm)
        self.current_llm = None

  


from antlr4 import ParseTreeListener
from agenterprise.agent_grammer.parser.ai_environmentListener import ai_environmentListener
from agenterprise.agent_grammer.parser.ai_environmentParser import ai_environmentParser
from agenterprise.model.data.ai_environment import AIEnvironment
from agenterprise.model.listener.AIURN import AIURN


class BasicServiceListener(ai_environmentListener):
    def __init__(self):
        self.app_pattern = None
        self.service_techlayer = None
        self.data_techlayer=None
        self.ai_techlayer = None
        self.agentic_middleware_techlayer = None
        self.environment = None

    def enterArchitectureAiStack(self, ctx):
        super().enterArchitectureAiStack(ctx)
        self.ai_techlayer = ctx.TECHLAYER_AIURN().getText()

    def enterArchitectureServiceStack(self, ctx):
        super().enterArchitectureServiceStack(ctx)
        self.service_techlayer = ctx.TECHLAYER_AIURN().getText()
    
    def enterArchitectureDataStack(self, ctx):
        super().enterArchitectureDataStack(ctx)
        self.data_techlayer = ctx.TECHLAYER_AIURN().getText()
    
    def enterArchitectureAgenticMiddlewareStack(self, ctx):
        super().enterArchitectureAgenticMiddlewareStack(ctx)
        self.agentic_middleware_techlayer = ctx.TECHLAYER_AIURN().getText()
 
    def exitEnvId(self, ctx):
        super().exitEnvId(ctx)
        self.envId = ctx.PROPERTYVALUE().getText()

    def exitAi_envDef(self, ctx):
        super().exitAi_envDef(ctx)
        self.environment = AIEnvironment(
            name=ctx.PROPERTYVALUE().getText(),
            ai_techlayer=AIURN(self.ai_techlayer),
            service_techlayer=AIURN(self.service_techlayer),
            data_techlayer=AIURN(self.data_techlayer),
            agentic_middleware_techlayer=AIURN(self.agentic_middleware_techlayer),
            envid=self.envId,
            agents=[],
            llms=[]
        )




from agenterprise.agent_grammer.parser.ai_environmentListener import  ai_environmentListener
from agenterprise.model.data.ai_environment import Agent
from agenterprise.model.listener.AIURN import AIURN

class BaseAIAgentListener(ai_environmentListener):
    def __init__(self):
        super().__init__()
        self.agents = []
        self.current_agent = None

    def enterAgentDef(self, ctx):
        super().enterAgentDef(ctx)
        self.current_agent = {
            "name": ctx.PROPERTYVALUE().getText(),
            "uid": None,
            "namespace": None,
            "systemprompt": None,
            "description": None,
            "examples": [],
            "tags": [],
            "toolrefs": [],
            "inputproperties": {},
            "outputproperties": {},
            "properties": {}
        }
    def enterAgentToolRefProperty(self, ctx):
        self.current_agent["toolrefs"].append(ctx.TOOLID().getText())
    def exitAgentDef(self, ctx):
        super().exitAgentDef(ctx)
        agent = Agent(
            name=self.current_agent.get("name"),
            uid=AIURN(self.current_agent.get("uid")),
            namespace=AIURN(self.current_agent.get("namespace")),
            systemprompt=self.current_agent.get("systemprompt"),
            description=self.current_agent.get("description", "No description"),
            tags=self.current_agent.get("tags", []),
            examples=self.current_agent.get("examples", []),
            llmref=AIURN(self.current_agent.get("llmref")),
            toolrefs=[ AIURN(x) for x in self.current_agent["toolrefs"]],
            properties=self.current_agent.get("properties", {}),
            output=self.current_agent.get("output"),
            input=self.current_agent.get("input")
        )
        self.agents.append(agent)
        self.current_agent = None

    def enterAgentIdentity(self, ctx):
        super().enterAgentIdentity(ctx)
        if self.current_agent is not None and ctx.AGENTID():
            self.current_agent["uid"] = ctx.AGENTID().getText()

    def enterAgentExampleProp(self, ctx):
        self.current_agent["examples"].append(ctx.PROPERTYVALUE().getText().strip('"'))
    
    def enterAgentTagsProp(self, ctx):
        self.current_agent["tags"].append(ctx.PROPERTYVALUE().getText().strip('"'))

    def enterAgentDescriptionProp(self, ctx):
        super().enterAgentDescriptionProp(ctx)
        if self.current_agent is not None:
            self.current_agent["description"] = ctx.PROPERTYVALUE().getText().strip('"')

    def enterAgentNamespace(self, ctx):
        super().enterAgentNamespace(ctx)
        if self.current_agent is not None and ctx.AGENTNAMESPACE():
            self.current_agent["namespace"] = ctx.AGENTNAMESPACE().getText()

    def enterAgentLLMRefProperty(self, ctx):
        super().enterAgentLLMRefProperty(ctx)
        if self.current_agent is not None and ctx.LLMID():
            self.current_agent["llmref"] = ctx.LLMID().getText()
   
    def enterAgentSystemPromptProperty(self, ctx):
        super().enterAgentSystemPromptProperty(ctx)
        if self.current_agent is not None:
            self.current_agent["systemprompt"] = ctx.PROPERTYVALUE().getText().strip('"')  

    def enterAgentInputProperty(self, ctx):
        super().enterAgentInputProperty(ctx)
        var = ctx.ENTITY_ID().getText()
        self.current_agent["input"]= AIURN(var)

    def enterAgentOutputProperty(self, ctx):
        super().enterAgentOutputProperty(ctx)
        var = ctx.ENTITY_ID().getText()
        self.current_agent["output"] = AIURN(var)

    def enterAgentCustomProperty(self, ctx):
        super().enterAgentCustomProperty(ctx)
        if self.current_agent is not None and ctx.VAR():
            key = AIURN(ctx.VAR().getText())
            value = ctx.PROPERTYVALUE().getText()
            self.current_agent["properties"][key] = value


import os
from typing import List
import uuid

from agenterprise.model.listener.AIURN import AIURN
from agenterprise.model.listener.agent.listener import BaseAIAgentListener
from agenterprise.model.listener.llm.listener import BaseAILLMListener
from agenterprise.model.listener.nonfunctional.listener import NonFunctionalListener
from agenterprise.model.listener.service.listener import BasicServiceListener
from agenterprise.model.listener.tool.listener import BaseAIToolListener
from agenterprise.model.listener.entity.listener import BaseAIEntityListener
from agenterprise.model.listener.agenticmiddleware.listener import BasicAgenticMiddlewareListener


class Project():

    def __init__(self, ai_techstack: AIURN, service_techstack: AIURN, data_techstack: AIURN, agentic_middleware_techstack: AIURN, target_dir: str, envid: str = None, dsl_file: str = None, llms_txt: str = None):
        self.ai_techstack = ai_techstack
        self.service_techstack = service_techstack
        self.data_techstack = data_techstack
        self.project_layer = service_techstack
        self.agentic_middleware_layer = agentic_middleware_techstack
        self.project_build_id = envid
        self.target_dir = target_dir
        self.dsl_file = dsl_file
        self.llms_txt = llms_txt

        self.projectlistener = NonFunctionalListener
        self.agentlistener = BaseAIAgentListener
        self.llmlistener = BaseAILLMListener
        self.servicelistener = BasicServiceListener
        self.toollistener = BaseAIToolListener
        self.entitylistener = BaseAIEntityListener
        self.agenticMiddlewareListener = BasicAgenticMiddlewareListener

    def get_projectlayer(self):
        return self.project_layer.to_url()
    def get_agentlayer(self):
        return self.ai_techstack.to_url()
    def get_servicelayer(self):
        return self.service_techstack.to_url()    
    def get_llmlayer(self):
        return self.ai_techstack.to_url()      
    def get_toollayer(self):
        return self.ai_techstack.to_url()    
    def get_entitylayer(self):
        return self.data_techstack.to_url()    
    def get_agentic_middleware_layer(self):
        return self.agentic_middleware_layer.to_url()    
    def get_dsl_filename(self):
        return os.path.basename(self.dsl_file) if self.dsl_file else None
